package org.jabber.applet;
/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ or at http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
 * This class is basically a JabberID with a few additional properties.
 * 
 * @author David Scott
 */
public class JID {
    /**
     * JID username
     */
    private String username = ""; // users jabber name

    /**
     * JID server
     */
    private String server = "";   // server to connect to

    /**
     * JID resource
     */
    private String resource = ""; // resource associated with current logon

    /**
     * status of user
     */
    private String status = "";   // current status of user

    /**
     * user's exact availability
     */
    private String show = "";	  // current availability of user

    /**
     * nickname of user
     */
    private String nick = "";     // friendly name

    /**
     * group associated with user
     */
    private String group = "";     // group name

    /**
     * bit masks for jid search criteria
     */
    public static int MSK_USERNAME = 1;
    public static int MSK_SERVER = 2;
    public static int MSK_RESOURCE = 4;
    public static int MSK_NICKNAME = 8;
    public static int MSK_GROUP = 16;

    /**
     * JID Constructor
     * @param username portion of JID
     * @param server portion of JID
     */
    public JID(String username, String server) {
        this (username, server, "", "", "", "unavailable");
    }
    /**
     * JID Constructor
     * @param username portion of JID
     * @param server portion of JID
     * @param nick nick/freindly displayed name
     */
    public JID(String username, String server, String nick) {
        this (username, server, nick, "", "", "unavailable");
    }
    /**
     * JID Constructor
     * @param username portion of JID
     * @param server portion of JID
     * @param nick nick/freindly displayed name
     * @param group portion of JID
     */
    public JID(String username, String server, String nick, String group) {
        this (username, server, nick, group, "", "unavailable");
    }

    /**
     * JID Constructor
     * @param sInput string to parse into JID
     * @param nick nick/freindly displayed name
     * @param group portion of JID
     * @param Ignore this is needed to make signature unique (ignored)
     */
    public JID(String sInput, String nick, String group, boolean Ignore) {
        //convert JID to name/server/resource using exisiting constructor
        this (sInput);

        //change to supplied values
        if (nick.length() != 0)
            this.nick =  nick;
        else
            this.nick = this.username;

        this.group = group;
    }

    /**
     * JID Constructor
     * @param username portion of JID
     * @param server portion of JID
     * @param nick nick/freindly displayed name
     * @param group portion of JID
     * @param resource portion of JID
     */
    public JID(String username, String server, String nick, String group, String resource, String show) {
        // set server
        if ((server==null) || (server.trim().equals("")))
            throw new RuntimeException("illegal null server");
        this.server=server;

        // set username and nickname
        if ((username==null) || (username.trim().equals("")))
            throw new RuntimeException("illegal username");

        this.username=username;//.toLowerCase();
        if (nick.length() > 0)
            this.nick = nick;
        else
            this.nick = username;

        // set resource and status
        this.group = group;
        this.resource=resource;
        this.show = show;
    }

    /**
     * JID Constructor using "user@server/resource" input string.
     * NOTE/WARNING: resource is used for nick in this case since we have none.
     * @param sInput string to parse into JID
     */
    public JID(String sInput) {
        String sUser = "";                   // JID user
        String sServer = "";                 // JID server
        String sResource = "";               // JID resource
        int i, j, k;

        // bypass 'jabber:' if it exists
        i = sInput.toUpperCase().indexOf("JABBER:");
        if (i > -1)
            i = 7;
        else
            i = 0;
        // extract username (preceding @)
        j = sInput.indexOf("@",i);
        if (j > -1)
            sUser = sInput.substring(i,j);
        else
            //sUser = "";
            sUser = sInput;
        this.username = sUser;//.toLowerCase();

        // extract sever/resource
        k = sInput.indexOf("/",j);
        if (k > -1)                     // found resource delim
        {                               // extract server & resource
            sServer = sInput.substring(j+1,k);
            sResource = sInput.substring(k+1);
        } else                            // no resource delim
        {                               // extract server, no resource
            sServer = sInput.substring(j+1);
            sResource = "";
        }
        this.server = sServer;
        this.resource = sResource;
        this.nick = sUser;
    }

    /**
     * get active status of user
     * @return if the user is active or not.
     */
    public boolean isActive() {
        return (!this.show.equals("unavailable"));
    }

    /**
     * Set status of user
     * @param str new status associated with user
     * @param show new sort of presence
     */
    public void setStatus(String str, String show) {
        this.status = str;
        this.show = show;
    }

    /**
     * Set group of user
     * @param str new group of user
     */
    public void setGroup(String str) {
        this.group = str;
    }

    /**
     * Set nickname of user
     * @param str new nickname of user
     */
    public void setNick(String str) {
        this.nick = str;
    }

    /**
     * Set users JID username
     * @param str username to set
     */
    public void setUsername(String str) {
        this.username = str;//.toLowerCase();
    }

    /**
     * Set JID server
     * @param str server to set
     */
    public void setServer(String str) {
        this.server = str;
    }

    /**
     * Get users status
     * @return Users current status
     */
    public String getStatus() {
        return(status);
    }

    /**
     * Get user's show attribute
     * @return User's show attribute
     */
    public String getShow() {
        return(show);
    }

    /**
     * Get JID
     * @return Users JID 
     */
    public String getJID() {
        return username + "@" + server;
    }

    /**
     * Get Username
     * @return Users JID username
     */
    public String getUsername() {
        return username;
    }

    /**
     * Get users nickname/friendly name
     * @return Users friendly name
     */
    public String getNick() {
        if (nick == "")
            return username;
        else
            return nick;
    }

    /**
     * Get server user is associated with
     * @return Jabber Server
     */
    public String getServer() {
        return server;
    }

    /**
     * Get group user is associated with
     * @return group
     */
    public String getGroup() {
        return group;
    }

    /**
     * Get Resource for this JID
     * @return JID resource
     */
    public String getResource() {
        return resource;
    }

    /**
     * Test if input params would create a valid JID
     * @return valid jid indication true/false
     */
    public static boolean validJID(String username, String server) {
        if ((username==null) || (username.trim().equals("")))
            return false;
        if ((server==null) || (server.trim().equals("")))
            return false;
        return true;
    }

    /**
     * Is JID = input JID
     * @param jid to compare to
     * @param nMask of which parts of jid to compare (username, server, resource, nick, group)
     * @return true if compared items are =, false if compared items aren't =
     */
    public boolean isEqual(JID jid, int nMask) {
        if (((nMask & MSK_USERNAME)>0) && !getUsername().equalsIgnoreCase(jid.getUsername()))
            return( false );
        if (((nMask & MSK_SERVER)>0) && !getServer().equalsIgnoreCase(jid.getServer()))
            return( false );
        if (((nMask & MSK_RESOURCE)>0) && !getResource().equals(jid.getResource())) // Resource is case sensitive!
            return( false );
        if (((nMask & MSK_NICKNAME)>0) && !getNick().equals(jid.getNick()))
            return( false );
        if (((nMask & MSK_GROUP)>0) && !getGroup().equals(jid.getGroup()))
            return( false );
        return( true );
    }

    /**
     * Convert JID to string
     * @return username@server/resource
     */
    public String toString() {
        return getUsername() + "@"+ getServer()+"/"+getResource();
    }
}
