package org.jabber.applet.awt;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import org.jabber.applet.*;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ or at http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
 * This class is used for a Jabber Roster List.
 * The list of users are displayed via a list of groups.
 * Within each group are n users.
 *
 * This class implements RosterListener - to update itself when the
 * data changes - and MouseListener - to update the model when the
 * user interacts - and ActionListener to update when the user 
 * selects a menu item.
 */
public class ListRosterComponent extends java.awt.List
    implements RosterListener, MouseListener, ActionListener {

    /**
     * Display groups with this in front
     */
    private final String groupExpandable = "+ ";
    private final String groupShrinkable = "-  ";

    /**
     * Labels used as menu bar.
     */
    private Label jabberMenu = new Label(" Jabber ");
    private Label userMenu = new Label(" User ");
    private Label helpMenu = new Label(" Help ");

    /**
     * Button to set status back to available.
     */
    private Button unavailable = new Button(" Set Status To " + Config.AVAIL);

    /**
     * Jabber popup menu
     */
    public static PopupMenu POPUP_JABBER;

    /**
     * User popup menu
     */
    public static PopupMenu POPUP_USER;

    /**
     * Help popup menu
     */
    public static PopupMenu POPUP_HELP;

    /**
     * MenuItems for logOn and logOff.
     */
    private MenuItem logOn;
    private MenuItem logOff;
    private MenuItem newAccount;
    private MenuItem advanced;

    /**
     * MenuItem for group chat.
     */
    private MenuItem groupMenuItem;

    /**
     * Reference to the main applet.
     */
    private static JabberApplet applet;

    /**
     * Panel to hold the menu labels.
     */
    private Panel available;

    /**
     * Container for the roster and menus.
     */
    private Container wrapper;

    /**
     * Data model for list.
     */
    JIDList model;

    /**
     * Show all roster users?
     */
    private boolean showAll = false;

    /**
     * Text for menu item to toggle showAll behavior.
     */
    private static String SHOW_ALL = "Show All Users";
    private static String SHOW_ACTIVE = "Show Online Users";

    /**
     * Vector to keep mapping of displayed elements to
     * their model indexes.
     */
    private Vector indexes = new Vector();

    /**
     * Constructor
     */
    public ListRosterComponent( JIDList model ) {
        super();
        applet = JabberApplet.getAppletInstance();
        model.addRosterListener(this);
        addMouseListener(this);
        this.model = model;
        createPopupMenu( );
    }

    /**
     * Roster list to reflect 'Available' status.
     * Put the menu bar back and enable the roster
     * component to listen for events.
     */
    public void setAvailable() {
        unavailable.setVisible(false);
        Container c = unavailable.getParent();
        if( c != null ) {
            c.remove(unavailable);
            c.validate();
            c.add(available,"North");
            c.validate();
            c.repaint();
        }
        setEnabled(true);
    }

    /**
     * Roster list to reflect 'Unavailable' status.
     * Remove the menu bar and disable the roster
     * component to listen for events.
     */
    public void setUnavailable() {
        Container c = available.getParent();
        if( c != null ) {
            unavailable.setBackground(available.getBackground());
            c.remove(available);
            c.add(unavailable,"North");
            unavailable.setVisible(true);
            wrapper.repaint();
            wrapper.validate();
        }
        setEnabled(false);
    }

    /**
     * Create a wrapper component that holds the
     * 'menu' and roster list.
     * @return component containing the 'menu' and roster list.
     */
    public Component getWrapper(boolean bmenu) {
        if( bmenu ) {
            wrapper = new Panel();

            wrapper.add(POPUP_JABBER);
            wrapper.add(POPUP_USER);
            wrapper.add(POPUP_HELP);

            wrapper.setLayout(new BorderLayout());

            jabberMenu.addMouseListener(this);
            userMenu.addMouseListener(this);
            helpMenu.addMouseListener(this);

            unavailable.addActionListener(this);

            jabberMenu.setFont(Config.CLIENT_FONT);
            userMenu.setFont(Config.CLIENT_FONT);
            helpMenu.setFont(Config.CLIENT_FONT);

            available= new Panel();

            FlowLayout fl = new FlowLayout(FlowLayout.LEFT);
            fl.setVgap(0);
            fl.setHgap(0);
            available.setLayout( fl );

            available.add(jabberMenu);
            available.add(userMenu);
            available.add(helpMenu);

            wrapper.add( available, "North" );
            wrapper.add( this, "Center" );

            return wrapper;
        } else {
            return this;
        }
    }


    /**
     * Implement RosterListener interface. Listen for roster changes.
     */
    public void rosterChanged( java.util.EventObject eo ) {
        display();
    }


    /**
     * Create/Display the roster list.
     */
    protected void display() {
        indexes.removeAllElements();
        removeAll();                        // clear current display
        for( int i = 0; i< model.getSize(); i++ ) {
            Object o = model.getElementAt(i);
            if( o instanceof JIDGroup ) {
                if( addGroup((JIDGroup)o) ) {
                    indexes.addElement( new Integer(i) );
                }
            } else if( o instanceof JID ) {
                if( addJID((JID)o) ) {
                    indexes.addElement( new Integer(i) );
                }
            } else if( o != null ) {
                add
                    ( o.toString() );
                indexes.addElement( new Integer(i) );
            }
        }
    }

    /**
     * Add Group to display.
     */
    private boolean addGroup( JIDGroup group ) {
        if( group == null )
            return false;
        add
            ( group.toString() );
        return true;
    }

    /**
     * Add JID to display.
     */
    private boolean addJID( JID jid ) {
        if( jid == null ||
                (!showAll && !jid.isActive()) )
            return false;
        String name = jid.getUsername();
        String nick = jid.getNick();
        String status = jid.getStatus();
        String show = jid.getShow();
        if( status != null && status.length() > 0 ) {
            if(status.equals("")) {
                nick += " <"+show+">";
            } else {
                nick += " <"+show+": "+status+">";
            }
        }
        add
            ( "    "+nick );
        return true;
    }

    /**
     * mouse pressed
     */
    public void mousePressed( MouseEvent me ) {
mouseClicked(  me );
}
/*        // do nothing if in menu.
        Component obj = me.getComponent();
        if( obj == jabberMenu ) {}
        else if( obj == userMenu ) {}
        else if( obj == helpMenu ) {}
        else if( me.isPopupTrigger() && model.isJIDSelected() ) {
            if( ! isEnabled() )
                return;
            groupMenuItem.setEnabled(false);
            popMe( POPUP_USER, obj, new Rectangle(40, me.getY(), 0, 10) );
        }
    }
nicht fuer ce*/
    /**
     * mouse released
     */
    public void mouseReleased( MouseEvent me ) {
mouseClicked(  me );
}
/*        // do nothing if in menu.
        Component obj = me.getComponent();
        if( obj == jabberMenu ) {}
        else if( obj == userMenu ) {}
        else if( obj == helpMenu ) {}
        else if( me.isPopupTrigger() && model.isJIDSelected() ) {
            if( ! isEnabled() )
                return;
            groupMenuItem.setEnabled(false);
            popMe( POPUP_USER, obj, new Rectangle(40, me.getY(), 0, 10) );
        }
    }
nicht fuer ce*/
    /**
     * mouse clicked
     */
    public void mouseClicked( MouseEvent me ) {
        Component obj = me.getComponent();
        if( obj == unavailable ) {
            setAvailable();
            applet.presenceSetAvail();
        }
        if( obj == jabberMenu ) {
            if( me.getModifiers() != Event.META_MASK ) {
                Rectangle r = new Rectangle(jabberMenu.getBounds());
                r.x = 0;
                popMe( POPUP_JABBER, jabberMenu, r );
            }
        } else if( obj == userMenu ) {
            if( me.getModifiers() != Event.META_MASK ) {
                Rectangle r = new Rectangle(userMenu.getBounds());
                r.x = 0;
                groupMenuItem.setEnabled(true);
                popMe( POPUP_USER, userMenu, r );
            }
        } else if( obj == helpMenu ) {
            if( me.getModifiers() != Event.META_MASK ) {
                Rectangle r = new Rectangle(helpMenu.getBounds());
                r.x = 0;
                popMe( POPUP_HELP, helpMenu, r );
            }
        } else if( me.isPopupTrigger() && model.isJIDSelected() ) {
            if( ! isEnabled() )
                return;
            groupMenuItem.setEnabled(false);
            popMe( POPUP_USER, obj, new Rectangle(40, me.getY(), 0, 10) );
        } else if( me.getClickCount() >= 2
                   && me.getModifiers() != Event.META_MASK ) {
            if( ! isEnabled() )
                return;
            // double click. expand/contract group or launch window.
            if( model.isJIDSelected() )
                applet.launchChat((JID)model.getSelectedItem());
            else if( model.isGroupSelected() )
                model.toggleGroup();
        } else if( me.getClickCount() <= 1
                   && me.getModifiers() != Event.META_MASK ) {
            if( ! isEnabled() )
                return;

            int index = getSelectedIndex();
            model.setSelectedIndex( getSelectedIndex(index) );
        }
    }
public void mouseEntered( MouseEvent me ) {}
    public void mouseExited( MouseEvent me ) {}


    /**
     * Get selected index 
     */
    private int getSelectedIndex( int offset ) {
        try {
            Integer index =
                (Integer)indexes.elementAt(offset);
            return  index.intValue();
        } catch( Exception e ) {
            return -1;
        }
    }

    /**
     * set properties for this component.
     * @param properties.
     */
    public void setProperties( Properties properties ) {}

    /**
    * Popup a menu below the parent rectangle
    * @param popMenu popup menu to use
    * @param msEvent mouse event
    * @param parentRect parent rectangle
    */
    private void popMe( PopupMenu popMenu, Component c, Rectangle parentRect) {
        if( applet.isLoggedOff() ) {
            logOn.setEnabled(true);
            logOff.setEnabled(false);
        } else {
            logOn.setEnabled(false);
            logOff.setEnabled(true);
        }
        if (applet.isGroupChatting()) {
            advanced.setEnabled(false);
        } else {
            advanced.setEnabled(true);
        }
        int x = parentRect.getLocation().x;     // set x same as parent
        int y = parentRect.getLocation().y;     // set y below parent
        y = y + parentRect.getSize().height;
        popMenu.show(c, x, y);
    }                                           // end, popme routine


    /**
    * Handle (most likely) mouse action
    */
    public void actionPerformed( ActionEvent e ) {
        if( e.getActionCommand().equals(" Set Status To " + Config.AVAIL) )
        {
            setAvailable();
            applet.presenceSetAvail();
        }
        if (e.getActionCommand() == Config.NEW_USER) {
            applet.newUserRegistration();
        } else if (e.getActionCommand() == Config.CREATE_NEW_ACCOUNT) {
            // user logoff jabber server to create a new account
            if( !applet.isLoggedOff() ) {
                applet.logOff();
            }
            // new window to collect the new user info
            applet.newAccountDialog(null);
        } else if (e.getActionCommand() == Config.SIGN_ON) {
            applet.logonDialog();                // user logon to jabber server
        } else if (e.getActionCommand() == Config.LOG_OFF) {
            applet.logOff();                   // user logoff jabber server
        } else if (e.getActionCommand() == Config.ADVANCED) {
            applet.advancedSettings();
        } else if (e.getActionCommand() == Config.HELP_HELP) {
            applet.launchHelp();
        } else if (e.getActionCommand() == Config.VERSION) {
            applet.displayVersion();
        } else if (e.getActionCommand() == Config.WHOAMI) {
            applet.displayWhoami();
        } else if (e.getActionCommand() == Config.AVAIL) {
            setAvailable();
            applet.presenceSetAvail();
        } else if (e.getActionCommand() == Config.UNAVAIL) {
            setUnavailable();
            applet.presenceSetUnavail();
        } else if (e.getActionCommand() == Config.GROUP) {
            applet.groupChatPrompt();
        } else if (e.getActionCommand() == Config.SEND_MSG) {
            applet.sendMessage();
        } else if (e.getActionCommand() == Config.ADD_USER) {
            applet.presenceSubscribeInfo();
        } else if (e.getActionCommand() == Config.EDIT_USER) {
            applet.presenceUpdateInfo();
        } else if (e.getActionCommand() == Config.REMOVE_USER) {
            applet.presenceUnsubscribeRequest();
        } else if( e.getActionCommand().equals(SHOW_ACTIVE) ) {
            showAll = false;
            model.setAllExpanded();
            try {
                MenuItem mi = (MenuItem)e.getSource();
                mi.setLabel(SHOW_ALL);
                mi.setActionCommand(SHOW_ALL);
                Object o = model.getSelectedItem();
                if( o != null && o instanceof JID
                        && !( ((JID)o).isActive() ) ) {
                    // selected item will no longer be displayed.
                    // reset selected to nothing.
                    model.setSelectedIndex( -1 );
                }
                display();
            } catch( Exception ex ) {}
        }
        else if( e.getActionCommand().equals(SHOW_ALL) ) {
            showAll = true;
            model.setAllExpanded();
            try {
                MenuItem mi = (MenuItem)e.getSource();
                mi.setLabel(SHOW_ACTIVE);
                mi.setActionCommand(SHOW_ACTIVE);
                display();
            } catch( Exception ex ) {}
        }
    }

    /**
     * Create Popup Menu.
     * Create the main toolbar popup menu(s) similar to the windows 
     * standard File, Edit, Help,,,.
     */
    private void createPopupMenu( ) {
        MenuItem mnu;

        // main jabber menu
        // avail, unavail, logoff
        POPUP_JABBER = new PopupMenu("Jabber");
        //POPUP_JABBER.add(mnu = new MenuItem (Config.AVAIL));
        //mnu.addActionListener (this);   // add Available
        POPUP_JABBER.add(mnu = new MenuItem (Config.UNAVAIL));
        mnu.addActionListener (this);   // add unavailable
        POPUP_JABBER.addSeparator();     // line separator

        newAccount = new MenuItem(Config.CREATE_NEW_ACCOUNT);
        POPUP_JABBER.add(newAccount);
        newAccount.addActionListener (this);   // add create new user
        POPUP_JABBER.addSeparator();     // line separator

        logOn = new MenuItem(Config.SIGN_ON);
        POPUP_JABBER.add(logOn);
        logOn.addActionListener (this);   // add logon

        logOff = new MenuItem(Config.LOG_OFF);
        POPUP_JABBER.add(logOff);
        logOff.addActionListener (this);   // add logoff
        POPUP_JABBER.addSeparator(); // line separator

        advanced = new MenuItem(Config.ADVANCED);
        POPUP_JABBER.add(advanced);
        advanced.addActionListener (this); // add Advanced Settings



        // user menu (message send, add, edit, remove)
        POPUP_USER = new PopupMenu("User");

        groupMenuItem = new MenuItem(Config.GROUP);
        POPUP_USER.add( groupMenuItem );
        groupMenuItem.addActionListener (this);

        POPUP_USER.add(mnu = new MenuItem(Config.SEND_MSG));
        mnu.addActionListener (this);
        POPUP_USER.addSeparator();       // line separator
        POPUP_USER.add(mnu = new MenuItem(Config.ADD_USER));
        mnu.addActionListener (this);
        POPUP_USER.add(mnu = new MenuItem(Config.EDIT_USER));
        mnu.addActionListener (this);
        POPUP_USER.add(mnu = new MenuItem(Config.REMOVE_USER));
        mnu.addActionListener (this);
        POPUP_USER.addSeparator();       // line separator
        if( showAll ) {
            POPUP_USER.add(mnu = new MenuItem(SHOW_ACTIVE));
            mnu.addActionListener (this);
        } else {
            POPUP_USER.add(mnu = new MenuItem(SHOW_ALL));
            mnu.addActionListener (this);
        }

        // help menu (help, who am i, version)
        POPUP_HELP = new PopupMenu("Help");
        String tmp = applet.getParameter("help");
        if(tmp != null && tmp.length() > 0) {
            POPUP_HELP.add(mnu = new MenuItem (Config.HELP_HELP));
            mnu.addActionListener (this);
        }
        POPUP_HELP.add(mnu = new MenuItem (Config.WHOAMI));
        mnu.addActionListener (this);
        POPUP_HELP.add(mnu = new MenuItem (Config.VERSION));
        mnu.addActionListener (this);

        applet.add(POPUP_JABBER);
        applet.add(POPUP_USER);
        applet.add(POPUP_HELP);
    }                                   // end, createPopupMenu
}
