package org.jabber.applet.communication;
import org.jabber.applet.tools.*;
/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ or at http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
 * Used to easily access InfoQuery <item> type parameters.
 * 
 * @author David Scott
 */
public class IQItem {
    /**
     * Username (1st part of the JID)
     */
    private String name = "";

    /**
     * server user is based
     */
    private String server = "";

    /**
     * none
     * from - they've subscribe to us
     * to - we've subscribed to them
     * both - to + from
     */
    private String subscription = "";

    /**
     * Type (subscription type (subscribe, unsubscribe,,,: 
     */
    private String type = "";

    /**
     * A user may belong to one (* or more *) groups.
     * note: currently only one group per user is supported.
     */
    private String group = "";

    /**
     * Nickname (friendly name) of user
     */
    private String nick = "";

    /**
     * May be one/none of the following:
     * subscribe - a request for a subscription is pending
     * unsubscribe - a request for an unsubscribe is pending
     */
    private String ask = "";

    /**
     * Create an InfoQuery Item given the JIDs name, server and type of item.
     * 
     * @param sName JID name
     * @param sServer JID server
     * @param sType type of Infoquery item
     */
    public IQItem(String sName, String sServer, String sType ) {
        this.name = sName;
        this.server = sServer;
        this.type = sType;
        this.nick = sName;
    }

    /**
     * Create an infoquery item given the name, server, nick group
     * 
     * @param sName JID name
     * @param sServer JID server
     * @param sNick JID nickname
     * @param sGroup roster group to place user
     */
    public IQItem(String sName, String sServer, String sNick, String sGroup) {
        // -------------------------------------
        // Extract jid username
        // -------------------------------------
        this.name = sName;
        this.server = sServer;
        this.nick = sNick;
        this.group = sGroup;
    }

    /**
     * Create an InfoQuery Item given a string containing misc info.
     * Most of the work is done in the constructor.  
     * Some or all of the subitems may be included in the string to be parsed.
     * 
     * @param sItem String describing the item
     */
    public IQItem(String sItem) {
        MiniXMLParser oParser = new MiniXMLParser();

        // -------------------------------------
        // Extract jid username and server name
        // -------------------------------------
        String sJID = oParser.parseTag(sItem, "jid");
        int j = sJID.indexOf("@");				// end of name
        if (j > -1) {
            this.name = sJID.substring(0,j);	// get name
            this.server = sJID.substring(j+1);	// get server
        } else {
            j = sJID.indexOf("/");				// agent item?
            if (j > -1) {
                this.server = sJID.substring(0,j);
                return;
            }
        }

        // -------------------------------------
        // Extract subscription
        // -------------------------------------
        this.subscription = oParser.parseTag(sItem, "subscription");

        // -------------------------------------
        // Extract ask
        // -------------------------------------
        this.ask = oParser.parseTag(sItem, "ask");

        // -------------------------------------
        // Extract nickname
        // -------------------------------------
        this.nick = oParser.parseTag(sItem, "name");

        // -------------------------------------
        // Extract jid group (default to friends)
        // -------------------------------------
        this.group = oParser.parseTag(sItem, "group");

        // If no group tag found, default to friends
        if (this.group.length() == 0)
            this.group = "Friends";
    }

    /**
     * Get username
     * 
     * @return JID username
     */
    public String getName() {
        return(name);
    }

    /**
     * Get server
     * 
     * @return JID server
     */
    public String getServer() {
        return (server);
    }

    /**
     * Get nickname
     * 
     * @return Users nickname/friendly name
     */
    public String getNick() {
        return (nick);
    }

    /**
     * Get subscription associated with user
     * 
     * @return Subscription type (to, from, both)
     */
    public String getSubscription() {
        return (subscription);
    }

    /**
     * Get group user is member of.
     * 
     * @return Group name
     */
    public String getGroup() {
        return (group);
    }

    /**
     * Get type associated with user
     * 
     * @return type
     */
    public String getType() {
        return (type);
    }

    /**
     * Get ask state associated with user
     * 
     * @return ask
     */
    public String getAsk() {
        return (ask);
    }

    /**
     * set nickname for user
     * 
     * @param str users nickname
     */
    public void setNick( String str ) {
        nick = str;
    }
    /**
     * set name for user
     * 
     * @param str users name
     */
    public void setName( String str ) {
        name = str;
    }
    /**
     * set group for user
     * 
     * @param str users group
     */
    public void setGroup( String str ) {
        group = str;
    }
}
