package org.jabber.applet.communication;

import org.jabber.applet.Config;
import org.jabber.applet.communication.Constants;
import org.jabber.applet.tools.SHA1;
import java.net.URL;
/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ 
 * 
 * or at
 * 
 * http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

import org.jabber.applet.connection.*;
import java.util.*;

/**
* This class handles only basic Jabber communication (meant to be small)
* 
* @author    Jonathan Paulson / Matthias Wimmer
* @version     $ 0.7.4 $
*/
public class JabberComm implements IConnectionClient {
    private IJabberClient       moClient            = null;
    private Connection          moConn              = null;
    private MiniJabberParser    moJabberParser      = null;
    private boolean                mbEchoXML        = false;
    private static final String STREAM_END          = "</stream:stream>";
    private static final String DEFAULT_HOST        = "jabber.org";
    private static final int    DEFAULT_PORT        = 5222;

    /**
     * Class Constructor
     *
     * @param client    Reference to the Object that implements IJabberClient 
     *                    Used to communicate back to the object 
     */
    public JabberComm(IJabberClient client) {
        moClient = client;
        moJabberParser = new MiniJabberParser(client);
    }


    /**
    * Set/reset the xml echo/debug flag
    * 
    * @param bDebug true=output xml, false=no debug output
    */
    public void setDebug(boolean bDebug) {
        mbEchoXML = bDebug;
    }

    //========= Required IConnectionClient methods
    /**
    * Raw Data received from host
    */
    public void dataReceived(String XML) {
        moClient.rawXML(XML);
        moJabberParser.ProcessJabber(XML, mbEchoXML);
    }

    /**
    * Communications Closed
    */
    public void connectionClosed() {
        moClient.connectionClosed();
        moClient.rawXML("\nDisconnected");
    }

    /**
    * Communications Error
    */
    public void error(String str) {
        moClient.rawXML(str);
        moClient.error(str);
    }
    //=========

    /**
    * Connect to specified Host/Port
    * 
    * @param host      Host Name or IP Address 
    * @param port      Port number
    * @return          Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int connect(String host, String hostName, int port, URL url) {
        String sXML = "";

        System.out.println("Connecting to " + host + " / " + hostName);

        //Create the Start packet XML String
        sXML ="<?xml version=\"1.0\" ?><stream:stream to=\"" + hostName
              + "\" xmlns=\"jabber:client\" xmlns:stream=\""
              + "http://etherx.jabber.org/streams\">";

        //Disconnect if already connected
        if (moConn != null) {
            moConn.disconnect();
        }

        //Open up communication pipe
        moConn = new ContinuousConnection(this);
        moConn.start();
        if (  moConn.connect(host, port).state() == ConnectionState.OPEN) {
            //Connect
            return sendRawXML(sXML);
        } else {
            // remove the not working connection
            moConn.cleanup();
            moConn=null;

            // try to get a polling connection
            if (url != null) {
                moConn = new PollingConnection(this, url);
                moConn.start();
                if (moConn.connect(host, port).state() == ConnectionState.OPEN) {
                    return sendRawXML(sXML);
                } else {
                    // remove the polling connection
                    moConn.cleanup();
                    moConn=null;
                    return Constants.UNABLE_TO_CONNECT;
                }
            } else {
                return Constants.UNABLE_TO_CONNECT;
            }
        }
    }

    /**
    * Disconnect from Jabber Server
    * 
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int disconnect() {
        if (moConn == null) {
            return Constants.NOT_CONNECTED;
        }
        int iReturn = sendRawXML(STREAM_END);
        moConn.disconnect();
        moConn.stop();
        return iReturn;
    }

    /**
    * Logon to Jabber Server
    * 
    * @param userName  UserName 
    * @param password  Password 
    * @param resource  Resource (ie work, home, etc.) 
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int logon(String userName, String password, String resource, String streamID) {
        try {
            SHA1 dSHA1 = new SHA1();
            dSHA1.init();
            dSHA1.updateASCII(streamID+password); // should be UTF-8!
            dSHA1.finish();
            String sXML = "<iq id=\""+Config.LOGON+"\" type=\"set\">"
                          + "<query xmlns=\"jabber:iq:auth\">"
                          + "<username>" + userName + "</username>"
                    /*      + "<digest>" + dSHA1.digout() + "</digest>" */
                          + "<password>" + password + "</password>" 
                          + "<resource>" + resource + "</resource></query></iq>";

            return sendRawXML(sXML);
        } catch(Exception e) {
            System.out.println(e);
        }
        return 0;
    }

    /**
    * Send xml that requests the creation of a new account to Jabber Server
    * 
    * @param userName  UserName 
    * @param password  Password 
    * @param resource  Resource (ie work, home, etc.) 
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int createNewAccount(String userName, String password, String resource) {
        String sXML = "<iq id=\""+Config.CREATE_NEW_ACCOUNT+"\" type=\"set\">"
                      + "<query xmlns=\"jabber:iq:register\">"
                      + "<username>" + userName + "</username>"
                      + "<password>" + password + "</password>"
                      + "<resource>" + resource + "</resource></query></iq>";

        return sendRawXML(sXML);
    }

    /**
    * Logoff from Jabber Server
    * 
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int logoff() {
        String sXML = STREAM_END;
        return sendRawXML(sXML);
    }

    /**
    * Set your Available (trueavailable or unavailable)
    * 
    * @param available True if available, false otherwise
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int setAvailable(boolean available) {
        String sStatus = "";
        if (available) {
            sStatus = "available";
        } else {
            sStatus = "unavailable";
        }
        String sXML = "<presence>type='" + sStatus + "'</presence>";

        return sendRawXML(sXML);
    }

    /**
    * Send Presence
    * 
    * @param to        Message To
    * @param from      Message From
    * @param type      Message Type
    * @param status    Message Status
    * @param priority  Message Priority
    * @param show      Show
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int sendPresence(String to, String from, String type,
                            String status, String priority, String show) {
        //Create Presence tag with only the info supplied (most of the fields are optional)
        String sXML = "<presence";
        if (to.length() > 0) {
            sXML += " to='" + to +"'";
        }
        if (from.length() > 0) {
            sXML += " from='" + from + "'";
        }
        if (type.length() > 0) {
            sXML += " type='" + type + "'";
        }

        sXML += ">";

        if (status.length() > 0) {
            sXML += "<status>" + status + "</status>";
        }
        if (priority.length() > 0) {
            sXML += "<priority>" + priority + "</priority>";
        }
        if (show.length() > 0) {
            sXML += "<show>" + show + "</show>";
        }

        sXML += "</presence>";

        return sendRawXML(sXML);
    }

    /**
    * Send Roster Request
    * 
    * @param resource   Resource
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int sendRosterRequest(String resource) {
        String status = "";
        String sXML = "<iq type=\"get\" id=\"" + resource + "\">"
                      + "<query xmlns=\"jabber:iq:roster\"/>"
                      + "</iq>";
        return sendRawXML(sXML);
    }


    /**
    * Send Roster Set
    * 
    * @param to         To
    * @param name       Name
    * @param group      Group
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int sendRosterSet(String to, String name, String group) {
        if (group == null) {
            //Default to friends
            group = "Friends";
        }

        String sXML = "<iq type=\"set\">"
                      + "<query xmlns=\"jabber:iq:roster\">"
                      + "<item jid=\"" + to + "\" name=\"" + name + "\"><group>" + group + "</group></item></query>"
                      + "</iq>";
        return sendRawXML(sXML);
    }

    /**
    * Send Unsubscribe Request
    * 
    * @param JID        JID        
    * @param name        Name
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int sendUnsubscribe(String JID, String name) {
        String sXML = "<iq type=\"set\">"
                      + "<query xmlns=\"jabber:iq:roster\">"
                      + "<item jid=\"" + JID + "\" name=\"" + name + "\" subscription=\"remove\"/>"
                      + "</query>"
                      + "</iq>";
        return sendRawXML(sXML);
    }

    /**
    * Send an Instant Message to a Jabber User
    * 
    * @param to            Message To
    * @param subject    Message Subject
    * @param body        Message Body
    * @param thread        Message Thread
    * @param type        Message Type
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */

    public int sendMsg(String to, String type, String subject, String body, String thread) {
        String sXML = "<message to='" + to + "'";

        //Create tag with only the info supplied (most of the fields are optional)

        if (type.length()>0) {
            sXML += " type='" + type + "'";
        }

        sXML += ">";

        if (subject.length() > 0) {  //This is a good check and <subject> won't send for a groupchat
            sXML += "<subject>" + subject + "</subject>";
        }
        if (body.length() > 0) {
            sXML += "<body>" + body + "</body>";
        }
        if (thread.length() > 0) {
            sXML += "<thread>" + thread + "</thread>";
        }

        sXML += "</message>";

        return sendRawXML(sXML);
    }

    /**
    * Notify recipient that you are starting to type a message
    * 
    * @param to            Who to notify (format: username@server/resource)
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    /*
    public int sendMsgReplyStarted(String to, String msg){
        String sXML = "<message to=\"" + to + "\">" 
                    + "<body></body>"
                    + "<x xmlns=\"jabber:x:event\"/><composing reply=\"" + msg + "\"/></message>";
        return sendRawXML(sXML);      
    }          
    */

    /**************************************************************/
    /**************************************************************/
    /******** G R O U P  C H A T **********************************/

    /**
    * Start the process to join a Group Chat Room
    * 
    * @param ID                Number used to help keep track of msg
    * @param chatRoom        Group Chat Room Name
    * @param chatServer        Chat Server Address
    *
    * @return                Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                        (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                        that the command succeeded, ie could still be an invalid login password)
    *                        Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int groupChatJoin(String ID, String chatRoom, String chatServer) {

        String sXML = "<iq type=\"get\" id=\"" + getID(ID) + "\" to=\"" + chatRoom + "@" + chatServer + "\">"
                      + "<query xmlns=\"jabber:iq:conference\"/></iq>"; //changed to iq:conference

        return sendRawXML(sXML);
    }

    /**
    * Request a Group Chat Nickname
    * 
    * @param ID                Number used to help keep track of msg
    * @param chatRoom        Group Chat Room Name
    * @param chatServer        Chat Server Address
    * @param nicknames        A comma delimited string with nickname candidates
    * @param key            A unique value sent by the server that is required to participate in chat
    * @param secret            Changes/Sets the secret word required to enter the group chat
    * @param privacy        True if you don't want your real JID to be sent to group members
    *
    * @return                Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                        (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                        that the command succeeded, ie could still be an invalid login password)
    *                        Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int groupChatNick(String ID, String chatRoom, String chatServer, String nicknames,
                             /*String key,*/ String secret, boolean privacy) {

        String sXML = "<iq type=\"get\" id=\"" + getID(ID) + "\" to=\"" + chatRoom + "@" + chatServer + "\">"
                      + "<query xmlns=\"jabber:iq:conference\">"; //changed to iq:conference

        StringTokenizer st = new StringTokenizer(nicknames,",");
        while (st.hasMoreTokens()) {
            sXML += "<nick>" + st.nextToken() + "</nick>";
        }

        //sXML +=  "<key>" + key + "</key>";

        if (secret.length() > 0) {
            sXML += "<secret>" + secret + "</secret>";
        }

        //        if (privacy){
        //            sXML += "<privacy/>";
        //        }

        sXML += "</query></iq>";

        return sendRawXML(sXML);
    }

    /**
    * Enter a Group Chat Room
    * 
    * @param chatRoom        Group Chat Room Name
    * @param chatServer        Chat Server Address
    * @param res            the unique never changing ID value that represents the nic
    *
    * @return                Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                        (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                        that the command succeeded, ie could still be an invalid login password)
    *                        Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int groupChatEnter(String ID, String chatRoom, String chatServer, String res, String nicknames) {
        String sXML = "<iq type=\"set\" id=\"" + getID(ID) + "\" to=\"" + chatRoom + "@" + chatServer + "\">"
                      + "<query xmlns=\"jabber:iq:conference\">";
        StringTokenizer st = new StringTokenizer(nicknames,",");
        while (st.hasMoreTokens()) {
            sXML += "<nick>" + st.nextToken() + "</nick>";
        }
        sXML += "</query></iq>";
        //Must Send Presence before Joining GroupChat Rooms
        if(sendPresence(chatRoom + "@" + chatServer + "/" + res, "", "", "", "", "") == Constants.SEND_SUCCEEDED) {
            return sendRawXML(sXML);
        } else {
            System.out.println("Trouble with groupChatEnter");
            return Constants.SEND_FAILED;
        }
    }

    /**
    * Leave a Group Chat Room
    * 
    * @param chatRoom        Group Chat Room Name
    * @param chatServer        Chat Server Address
    * @param res            the unique never changing ID value that represents the nic
    *
    * @return                Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                        (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                        that the command succeeded, ie could still be an invalid login password)
    *                        Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int groupChatLeave(String chatRoom, String chatServer, String res) {
        //To leave a group and stop receiving messages from it's participants, simply send an unavailable presence.
        //<presence type="unavailable" from="act3@gc.denmark/[res]"/>
        return sendPresence(chatRoom + "@" + chatServer + "/" + res, "", "unavailable", "", "", "");
    }

    /**
    * Subscscribe to a Group  Chat
    * 
    * @param chatRoom        Group Chat Room Name
    * @param chatServer        Chat Server Address
    * @param res            the unique never changing ID value that represents the nic
    *
    * @return                Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                        (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                        that the command succeeded, ie could still be an invalid login password)
    *                        Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int groupChatSubscribe(String chatRoom, String chatServer, String res) {
        //Groups can be subscribed to and exist in the user's roster. The address that should be subscribed to is
        //the same as the address for sending presence:
        //[group name]@[group chat server]/[res]
        return sendPresence(chatRoom + "@" + chatServer + "/" + res, "", "", "", "", "");
    }

    /**
    * Invite someone in to Group Chat Room
    * 
    * @param chatRoom        Group Chat Room Name
    * @param chatServer        Chat Server Address
    * @param toJID            The JID of the person you want to invite
    * @param inviteMsg        The invitation message to display
    *
    * @return                Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                        (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                        that the command succeeded, ie could still be an invalid login password)
    *                        Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int groupChatInvite(String chatRoom, String chatServer, String toJID, String inviteMsg) {
        //Anyone can ask the group chat server to invite someone else into the group by sending a message
        //to just the groupname@groupchatserver with a special subject in the format:
        //invite:[user to invite]@[user server]
        return sendMsg(chatRoom + "@" + chatServer, "", "invite:" + toJID, inviteMsg, "");
    }

    /**
    * A message to broadcast the the participants in the Group Chat
    * 
    * @param chatRoom        Group Chat Room Name
    * @param chatServer        Chat Server Address
    * @param subject        If specified, will change the topic of the group chat
    * @param msg            The message to broadcast the the group
    *
    * @return                Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                        (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                        that the command succeeded, ie could still be an invalid login password)
    *                        Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int groupChatMsg(String chatRoom, String chatServer, String subject, String msg) {
        subject = ""; //Can't send a subject based on iq:conference
        return sendMsg(chatRoom + "@" + chatServer, "groupchat", subject, msg, "");
    }

    /**
    * 
    * 
    * @param chatRoom        Group Chat Room Name
    * @param chatServer        Chat Server Address
    * @param subject        If specified, will change the topic of the group chat
    * @param msg            The private message to send to the specified user (represented by the res)
    * @param res            The res representing the person to send the private msg to
    *
    * @return                Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                        (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                        that the command succeeded, ie could still be an invalid login password)
    *                        Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int groupChatMsgPrivate(String chatRoom, String chatServer, String subject, String msg, String toRes) {
        /*
        <message type="chat" to="act3@gc.denmark/Ophelia">
          <body>
            Soft you now!
            The fair Ophelia! Nymph, in thy orisons
            Be all my sins remember'd
          </body>
        </message>
        */
        return sendMsg(chatRoom + "@" + chatServer + "/" + toRes, "chat", subject, msg, "");
    }

    /**************************************************************/
    /**************************************************************/
    /**************************************************************/

    public int sendRosterSet(String to, String name, String group, String ask) {
        if (group == null)
            group = "Friends";

        String sXML = "<iq type=\"set\">"
                      + "<query xmlns=\"jabber:iq:roster\">"
                      + "<item jid=\"" + to + "\" name=\"" + name + "\" ask=\"" + ask + "\"><group>" + group + "</group></item></query>"
                      + "</iq>";
        return sendRawXML(sXML);
    }

    public int sendAgentsRequest(String to, String id) {
        String sXML = "<iq id=\"" + id + "\" type=\"get\" to=\"" + to + "\">"
                      + "<query xmlns=\"jabber:iq:agents\"/>"
                      + "</iq>";
        return sendRawXML(sXML);
    }


    /**
    * Sends the supplied XML (without modifing it in any way) to the 
    * Jabber Server.  This allows developers using JabberAnt to use the latest
    * and greatest Jabber commands that aren't currently supported by JabberAnt.
    * This will also prove useful for testing.
    * 
    * @param XML        The EXACT XML to send to the server
    * @return           Constants.SEND_SUCCEEDED if the XML was trasmitted successfuly
    *                    (this indicates that the TRANSMISSION was successful, it DOESN'T indicate 
    *                     that the command succeeded, ie could still be an invalid login password)
    *                    Constants.SEND_FAILED if unable to send the XML to the server
    */
    public int sendRawXML(String XML) {
        if (moConn == null) {
            System.out.println("JabberComm.sendRawXML() called but not connected");
            return Constants.NOT_CONNECTED;
        }

        if (mbEchoXML)
            System.out.println("SENT: " + XML);
        boolean bRet = moConn.send(XML);
        if(mbEchoXML) {
            moClient.rawXML("\n==>\n" + XML + "\n<==\n");
        }
        if( bRet == true) {
            return Constants.SEND_SUCCEEDED;
        } else {
            System.out.println("JabberComm.sendRawXML() failed");
            return Constants.SEND_FAILED;
        }
    }


    //==================================================================
    //                P R I V A T E    M E T H O D S
    //==================================================================
    private String getID(String ID) {
        if (ID.length() > 0) {
            return ID;
        } else {
            //No ID supplied, so use a random one
            return "" + (int) ((Math.random() * 10000) + 1000);
        }
    }

}
